(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory(require('babel-runtime/core-js/get-iterator'), require('babel-runtime/helpers/classCallCheck'), require('babel-runtime/helpers/createClass'), require('babel-runtime/helpers/defineProperty'), require('babel-runtime/helpers/extends'), require('babel-runtime/core-js/object/assign'), require('babel-runtime/core-js/object/keys'), require('babel-runtime/helpers/typeof')) :
  typeof define === 'function' && define.amd ? define(['babel-runtime/core-js/get-iterator', 'babel-runtime/helpers/classCallCheck', 'babel-runtime/helpers/createClass', 'babel-runtime/helpers/defineProperty', 'babel-runtime/helpers/extends', 'babel-runtime/core-js/object/assign', 'babel-runtime/core-js/object/keys', 'babel-runtime/helpers/typeof'], factory) :
  (global.js-immutable = factory(global._getIterator,global._classCallCheck,global._createClass,global._defineProperty,global._extends,global._Object$assign,global._Object$keys,global._typeof));
}(this, (function (_getIterator,_classCallCheck,_createClass,_defineProperty,_extends,_Object$assign,_Object$keys,_typeof) { 'use strict';

_getIterator = 'default' in _getIterator ? _getIterator['default'] : _getIterator;
_classCallCheck = 'default' in _classCallCheck ? _classCallCheck['default'] : _classCallCheck;
_createClass = 'default' in _createClass ? _createClass['default'] : _createClass;
_defineProperty = 'default' in _defineProperty ? _defineProperty['default'] : _defineProperty;
_extends = 'default' in _extends ? _extends['default'] : _extends;
_Object$assign = 'default' in _Object$assign ? _Object$assign['default'] : _Object$assign;
_Object$keys = 'default' in _Object$keys ? _Object$keys['default'] : _Object$keys;
_typeof = 'default' in _typeof ? _typeof['default'] : _typeof;

/**
 * React State Reducer
 *
 * Copyright © 2015-2016 Robus Gauli, LLC. All rights reserved.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE.txt file in the root directory of this source tree.
 */

/**
 * Accepts the selector object and operation intended
 *
 * ### Example (es module)
 * ```js
 * import reduce from 'react-state-reducer';
 * const deepObject = {
 *  person: {
 *    name: 'robus gauli',
 *    address: {
 *      permanent: 'Nepal',
 *      temporary: 'China'
 *    }
 *  }
 * }
 *
 * const selector = {
 *  person: {
 *    address: {
 *      permanent: '#'
 *    }
 *  }
 * }
 *
 * const reducer = reduce(selector)
 * const result = reducer(deepObject)
 *                  .set('Holy land')
 *                  .apply()
 * // => {
 *    person: {
 *      name: ...,
 *      address: {
 *        permanent: 'Holy land',
 *        temporary: 'China'
 *      }
 *    }
 *  }
 * ```
 */

function reduce(selector) {
  if (selector === null || !((typeof selector === 'undefined' ? 'undefined' : _typeof(selector)) === 'object') || Array.isArray(selector)) {
    throw new Error('Invalid selector');
  }

  if (!_Object$keys(selector).length) {
    throw new Error('Invalid selector. No keys to traverse.');
  }
  return function selectorWrapper(_originalObject) {
    function set(oldValue, newValue) {
      if (newValue === null || newValue === undefined) {
        return oldValue;
      }
      return newValue;
    }

    function extend(oldArray, newArray) {
      if (!Array.isArray(newArray) || !Array.isArray(oldArray)) {
        return oldArray;
      }
      if (newArray.length) {
        return oldArray.concat(newArray);
      }
      return oldArray;
    }

    function append(oldList, value) {
      // if value is null or undefined return the same original list
      if (value === undefined || value === null || !Array.isArray(oldList)) {
        return oldList;
      }
      return oldList.concat(value);
    }

    function merge(oldObject, newObject) {
      if (newObject === null || newObject === undefined || (typeof newObject === 'undefined' ? 'undefined' : _typeof(newObject)) !== 'object') {
        return oldObject;
      }
      if ((typeof newObject === 'undefined' ? 'undefined' : _typeof(newObject)) === 'object' && !_Object$keys(newObject).length) {
        return oldObject;
      }
      return _Object$assign({}, oldObject, newObject);
    }

    // delete being the builtin operator
    function deleteOp(oldObject, key) {
      if (key === null || key === undefined) {
        return oldObject;
      }
      if (Array.isArray(oldObject)) {
        return oldObject.filter(function (val, index) {
          return key !== index;
        });
      }
      if ((typeof oldObject === 'undefined' ? 'undefined' : _typeof(oldObject)) === 'object' && oldObject !== null) {
        return _Object$keys(oldObject).reduce(function (acc, k) {
          return k === key ? _extends({}, acc) : _extends({}, acc, _defineProperty({}, k, oldObject[k]));
        }, {});
      }
      return oldObject;
    }

    var operationalMapper = {
      set: set,
      append: append,
      merge: merge,
      extend: extend,
      delete: deleteOp
    };

    var toUpdate = {};
    var targetPointer = '#';

    return new (function () {
      function _class() {
        _classCallCheck(this, _class);

        this.set = this.operationFactory('set');
        this.merge = this.operationFactory('merge');
        this.delete = this.operationFactory('delete');
        this.append = this.operationFactory('append');
        this.extend = this.operationFactory('extend');
      }

      _createClass(_class, [{
        key: 'of',
        value: function of(target) {
          if (!target) {
            return this;
          }
          if (typeof target === 'string' && target.startsWith('#')) {
            targetPointer = target;
          }
          return this;
        }
      }, {
        key: 'operationFactory',
        value: function operationFactory(operationName) {
          return function wrapper(value) {
            if (targetPointer !== '#') {
              toUpdate = _extends({}, toUpdate, _defineProperty({}, targetPointer, {
                value: value,
                operation: operationName
              }));
            } else if ((value !== undefined || value !== null) && targetPointer === '#') {
              toUpdate = _extends({}, toUpdate, {
                default: {
                  value: value,
                  operation: operationName
                }
              });
            }
            return this;
          }.bind(this);
        }
      }, {
        key: 'selectTransform',
        value: function selectTransform(originalObject, selectorObject) {
          if (typeof selectorObject === 'string' && selectorObject.startsWith('#')) {
            // we reached to the target
            var updateKeys = _Object$keys(toUpdate);

            if (!updateKeys.length) {
              throw new Error('Should at least has a one operation intended before applying changes');
            }

            if (selectorObject === '#') {
              if (updateKeys.indexOf('default') !== -1) {
                var updater = toUpdate.default;
                return operationalMapper[updater.operation](originalObject, updater.value);
              }
              return originalObject;
            } else if (updateKeys.indexOf(selectorObject) !== -1) {
              var _updater = toUpdate[selectorObject];
              return operationalMapper[_updater.operation](originalObject, _updater.value);
            }
            return originalObject;
          }

          var clonedObject = _Object$assign({}, originalObject);

          var _iteratorNormalCompletion = true;
          var _didIteratorError = false;
          var _iteratorError = undefined;

          try {
            for (var _iterator = _getIterator(_Object$keys(selectorObject)), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
              var key = _step.value;

              var keyedObject = clonedObject[key];
              if (keyedObject === undefined || keyedObject === null) {
                clonedObject = originalObject;
                break;
              }
              var filteredObject = selectorObject[key];
              clonedObject[key] = this.selectTransform(keyedObject, filteredObject);
            }
          } catch (err) {
            _didIteratorError = true;
            _iteratorError = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
              }
            } finally {
              if (_didIteratorError) {
                throw _iteratorError;
              }
            }
          }

          return clonedObject;
        }
      }, {
        key: 'apply',
        value: function apply() {
          return this.selectTransform(_originalObject, selector);
        }
      }]);

      return _class;
    }())();
  };
}

return reduce;

})));
//# sourceMappingURL=index.umd.js.map
